#include <stdio.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <signal.h>

#include <rtai_shm.h>
#include <rtai_nam2num.h>

#include <math.h>
#include "hinf.h"
#include "common.h"

static float * time_resp;
static double * cont_real;
static struct my_msg_struct * msg;
static int *pos_ref;

static void
emergency_stop(int d)
{
        msg->command = STOP_TASK;
	rtai_free(nam2num("time_resp"), time_resp);
	rtai_free(nam2num("cont_real"), cont_real);
	rtai_free(nam2num("msg"), msg);

	exit(1);
}

int
main(int argc, char *argv[])
{
	int i, j, count;
	FILE *fp, *fp2, *fp3;
	volatile double *p;

	fp = fopen("data/cont.dat", "r");
	if(fp == NULL){
		fprintf(stderr, "cannot open cont.dat\n");
		exit(1);
	}

        fp2 = fopen("data/cont_order.dat", "r");
	if(fp2 == NULL){
		fprintf(stderr, "cannot open cont_order.dat\n");
		exit(1);
	}

	fp3 = fopen("data/result.dat", "w");
	if(fp3 == NULL){
		fprintf(stderr, "cannot open result.dat\n");
		exit(1);
	}

	if((time_resp = (float *)rtai_malloc(nam2num("time_resp"), 0)) == NULL){
		fprintf(stderr, "rtai_malloc() failed for time_resp\n");
		return -1;
	}

	if((msg = (struct my_msg_struct *)rtai_malloc(nam2num("msg"), 0)) == NULL){
		fprintf(stderr, "rtai_malloc failed for msg\n");
		return -1;
	}

        if((pos_ref = (int *)rtai_malloc(nam2num("pos_ref"), 0)) == NULL){
                fprintf(stderr, "rtai_malloc failed \n");
                return -1;
        }

	msg->command = STOP_TASK;
	//	*pos_ref = PULSE_OFFSET;
	//	*pos_ref = PULSE_OFFSET - (2.64-V_OFFSET)*V2THETA*0.02*X2PULSE;
	//*pos_ref = PULSE_OFFSET - 500; // start from -5mm
	//sleep(5);

	signal(SIGINT, emergency_stop);

	fscanf(fp, "%lf", &msg->sampling_period);

	fscanf(fp2, "%d", &msg->order);
	fprintf(stderr, "order of the controller = %d\n", msg->order);

	if((cont_real = (double *)rtai_malloc(nam2num("cont_real"), 0)) == NULL){
		fprintf(stderr, "rtai_malloc failed for cont_real\n");
		return -1;
	}

	p = cont_real;
	for(i = 0; i < msg->order + 1; i++){
		for(j = 0; j < msg->order + 1; j++){
			fscanf(fp, "%lf", p);
			p++;
		}
	}

	time_resp[BUF_LEN * (RECORDING_SAMPLES - 1)] = -1.;

	/* Now start the tasks */
        fprintf(stderr, "Starting real time task ... ");
	msg->command = START_TASK;
	fprintf(stderr, "done.\n");

        fprintf(stderr, "Waiting for record ... ");
	while (time_resp[BUF_LEN * (RECORDING_SAMPLES - 1)] < 0.) fprintf(stderr, "val = %g\r ", time_resp[BUF_LEN * (RECORDING_SAMPLES - 1)]);
	fprintf(stderr, "done.\n");

	count = 0;
	for(i = 0; i < RECORDING_SAMPLES; i++){
		for(j = 0; j < BUF_LEN; j++){
			fprintf(fp3, "%g ", time_resp[count++]);
		}
		fprintf(fp3, "\n");
	}

	*pos_ref = 0;

	rtai_free(nam2num("time_resp"), time_resp);
	rtai_free(nam2num("cont_real"), cont_real);
	rtai_free(nam2num("msg"), msg);
	rtai_free(nam2num("pos_ref"), pos_ref);

	fclose(fp);
	fclose(fp2);
	fclose(fp3);

	return 0;
}
